from bioservices.eutils import EUtils
from bioservices.uniprot import UniProt
from xml.etree.ElementTree import register_namespace
import requests

def get_pubmed_info(pmid):
    eu = EUtils()
    r = eu.EFetch('pubmed', pmid, retmode='dict', rettype='abstract')
    article = r['PubmedArticleSet']['PubmedArticle']['MedlineCitation']['Article']
    title = article['ArticleTitle']
    authors_list = [a['LastName']+ ' ' + a['Initials'] for a in article['AuthorList']['Author']]
    authors = ', '.join(authors_list)
    journal_name = article['Journal']['Title']
    biblio_date = article['Journal']['JournalIssue']['PubDate']
    if 'Year' in biblio_date:
        biblio_year = biblio_date['Year']
    else:
        biblio_year = biblio_date['MedlineDate'][0:3]
    return {'title':title,
            'journal_name':journal_name,
            'biblio_year':biblio_year,
            'authors_list': authors}

def get_epo_info(patent_number):
    resp = requests.get('http://ops.epo.org/3.1/rest-services/published-data/publication/docdb/{}/biblio.json'.format(patent_number))
    data = resp.json()
    exchange_doc = data['ops:world-patent-data']['exchange-documents']['exchange-document']
    if isinstance(exchange_doc, list):
        exchange_doc = exchange_doc[0]
    title = [el['$'] for el in exchange_doc['bibliographic-data']['invention-title'] if el['@lang']=='en']
    authors = [i['inventor-name']['name']['$'] for i in exchange_doc['bibliographic-data']['parties']['inventors']['inventor'] if i['@data-format']=='original']
    biblio_year = [el['date']['$'][:4] for el in exchange_doc['bibliographic-data']['publication-reference']['document-id'] if el['@document-id-type']=='epodoc'][0]
    return {'title': title,
            'journal_name': None,
            'biblio_year': biblio_year,
            'authors_list': authors}

def get_uniprot_info(uniprot_id):
    uniprot_client = UniProt()
    resp = uniprot_client.retrieve(uniprot_id)
    recommended_name = resp.root.findall('{http://uniprot.org/uniprot}entry/{http://uniprot.org/uniprot}protein/{http://uniprot.org/uniprot}recommendedName/{http://uniprot.org/uniprot}fullName')[0].text
    organism = resp.root.findall('{http://uniprot.org/uniprot}entry/{http://uniprot.org/uniprot}organism/{http://uniprot.org/uniprot}dbReference[@type="NCBI Taxonomy"]')[0].attrib['id']
    gene = resp.root.findall('{http://uniprot.org/uniprot}entry/{http://uniprot.org/uniprot}gene/{http://uniprot.org/uniprot}name[@type="primary"]')[0].text
    return {'recommended_name': recommended_name,
            'organism': int(organism),
            'gene': gene
           }

def get_taxonomy_info(taxonomy_id):
    eu = EUtils()
    r = eu.EFetch('taxonomy', taxonomy_id, retmode='dict')
    scientific_name = r['TaxaSet']['Taxon']['ScientificName']
    return {'scientific_name': scientific_name} 
